@@ -22,6 +22,7 @@ import calendar
 import time
 import webbrowser
 from itertools import groupby
+import locale
 
 from gettext import ngettext
 
@@ -48,30 +49,27 @@ class TotalsBox(gtk.VBox):
         x_offset = 100 # align all graphs to the left edge
 
 
-        self.category_chart = charting.HorizontalBarChart(max_bar_width = 20,
-                                                          legend_width = x_offset,
-                                                          value_format = "%.1f",
-                                                          interactive = True)
+        self.category_chart = charting.Chart(max_bar_width = 20,
+                                             legend_width = x_offset,
+                                             value_format = "%.1f")
         self.category_chart.connect("bar-clicked", self.on_category_clicked)
         self.selected_categories = []
         self.category_sums = None
 
         self.get_widget("totals_by_category").add(self.category_chart);
 
-        self.activity_chart = charting.HorizontalBarChart(max_bar_width = 20,
-                                                          legend_width = x_offset,
-                                                          value_format = "%.1f",
-                                                          interactive = True)
+        self.activity_chart = charting.Chart(max_bar_width = 20,
+                                             legend_width = x_offset,
+                                             value_format = "%.1f")
         self.activity_chart.connect("bar-clicked", self.on_activity_clicked)
         self.selected_activities = []
         self.activity_sums = None
 
         self.get_widget("totals_by_activity").add(self.activity_chart);
 
-        self.tag_chart = charting.HorizontalBarChart(max_bar_width = 20,
-                                                     legend_width = x_offset,
-                                                     value_format = "%.1f",
-                                                     interactive = True)
+        self.tag_chart = charting.Chart(max_bar_width = 20,
+                                        legend_width = x_offset,
+                                        value_format = "%.1f")
         self.tag_chart.connect("bar-clicked", self.on_tag_clicked)
         self.selected_tags = []
         self.tag_sums = None
@@ -83,42 +81,43 @@ class TotalsBox(gtk.VBox):
         self.report_chooser = None
 
 
-    def on_category_clicked(self, widget, idx):
-        if idx in self.category_chart.bars_selected:
-            self.category_chart.bars_selected.remove(idx)
-            self.selected_categories.remove(self.category_sums[0][idx])
+    def on_category_clicked(self, widget, key):
+        if key in self.category_chart.selected_keys:
+            self.category_chart.selected_keys.remove(key)
+            self.selected_categories.remove(key)
         else:
-            self.category_chart.bars_selected.append(idx)
-            self.selected_categories.append(self.category_sums[0][idx])
+            self.category_chart.selected_keys.append(key)
+            self.selected_categories.append(key)
+
+        self.calculate_totals()
         self.do_charts()
 
-    def on_activity_clicked(self, widget, idx):
-        if idx in self.activity_chart.bars_selected:
-            self.activity_chart.bars_selected.remove(idx)
-            self.selected_activities.remove(self.activity_sums[0][idx])
+    def on_activity_clicked(self, widget, key):
+        if key in self.activity_chart.selected_keys:
+            self.activity_chart.selected_keys.remove(key)
+            self.selected_activities.remove(key)
         else:
-            self.activity_chart.bars_selected.append(idx)
-            self.selected_activities.append(self.activity_sums[0][idx])
+            self.activity_chart.selected_keys.append(key)
+            self.selected_activities.append(key)
+        self.calculate_totals()
         self.do_charts()
 
-    def on_tag_clicked(self, widget, idx):
-        if idx in self.tag_chart.bars_selected:
-            self.tag_chart.bars_selected.remove(idx)
-            self.selected_tags.remove(self.tag_sums[0][idx])
+    def on_tag_clicked(self, widget, key):
+        if key in self.tag_chart.selected_keys:
+            self.tag_chart.selected_keys.remove(key)
+            self.selected_tags.remove(key)
         else:
-            self.tag_chart.bars_selected.append(idx)
-            self.selected_tags.append(self.tag_sums[0][idx])
-        self.do_charts()
-
-
-    def on_reports_box_expose_event(self, box, someth):
+            self.tag_chart.selected_keys.append(key)
+            self.selected_tags.append(key)
+        self.calculate_totals()
         self.do_charts()
+        
 
     def search(self, start_date, end_date, facts):
         self.facts = facts
         self.category_sums, self.activity_sums, self.tag_sums = [], [], []
         self.selected_categories, self.selected_activities, self.selected_tags = [], [], []
-        self.category_chart.bars_selected, self.activity_chart.bars_selected, self.tag_chart.bars_selected = [], [], []
+        self.category_chart.selected_keys, self.activity_chart.selected_keys, self.tag_chart.selected_keys = [], [], []
 
         self.start_date = start_date
         self.end_date = end_date
@@ -131,6 +130,7 @@ class TotalsBox(gtk.VBox):
             self.get_widget("no_data_label").hide()
             self.get_widget("charts").show()
             self.get_widget("total_hours").show()
+            self.calculate_totals()
             self.do_charts()
         else:
             self.get_widget("no_data_label").show()
@@ -138,7 +138,7 @@ class TotalsBox(gtk.VBox):
             self.get_widget("total_hours").hide()
 
 
-    def do_charts(self):
+    def calculate_totals(self):
         if not self.facts:
             return
 
@@ -146,7 +146,7 @@ class TotalsBox(gtk.VBox):
         facts = copy.deepcopy(self.facts)
 
         total_hours = sum([stuff.duration_minutes(fact["delta"]) for fact in facts])
-        total_label = _("%s hours tracked total") % ("%.1f" % (total_hours / 60.0))
+        total_label = _("%s hours tracked total") % locale.format("%.1f", total_hours / 60.0)
         self.get_widget("total_hours").set_text(total_label)
 
 
@@ -203,6 +203,7 @@ class TotalsBox(gtk.VBox):
             self.tag_sums = zip(*tag_sums)
 
 
+    def do_charts(self):
         self.get_widget("totals_by_category").set_size_request(10,10)
         if self.category_sums:
             self.get_widget("totals_by_category").set_size_request(280, len(self.category_sums[0]) * 20)
