@@ -51,7 +51,7 @@ class CategoryStore(gtk.ListStore):
         """ Loads activity list from database, ordered by
             activity_order """
 
-        category_list = runtime.storage.get_category_list()
+        category_list = runtime.storage.get_categories()
 
         for category in category_list:
             self.append([category['id'],
@@ -75,7 +75,7 @@ class ActivityStore(gtk.ListStore):
         if category_id is None:
             return
 
-        activity_list = runtime.storage.get_activities(category_id)
+        activity_list = runtime.storage.get_category_activities(category_id)
 
         for activity in activity_list:
             self.append([activity['id'],
@@ -94,7 +94,7 @@ appearances = ["text", "icon", "both"]
 
 from configuration import runtime, conf
 import widgets
-import dispatcher, storage, stuff
+import storage, stuff
 
 class PreferencesEditor:
     TARGETS = [
@@ -108,6 +108,15 @@ class PreferencesEditor:
         self._gui = stuff.load_ui_file("preferences.ui")
         self.window = self.get_widget('preferences_window')
 
+        self.activities_sources = [("", _("None")),
+                                   ("evo", "Evolution"),
+                                   ("gtg", "Getting Things Gnome")]
+        self.todo_combo = gtk.combo_box_new_text()
+        for code, label in self.activities_sources:
+            self.todo_combo.append_text(label)
+        self.todo_combo.connect("changed", self.on_todo_combo_changed)
+        self.get_widget("todo_pick").add(self.todo_combo)
+
 
         # create and fill activity tree
         self.activity_tree = self.get_widget('activity_list')
@@ -233,6 +242,10 @@ class PreferencesEditor:
         self._gui.connect_signals(self)
         self.window.show_all()
 
+
+    def on_todo_combo_changed(self, combo):
+        conf.set("activities_source", self.activities_sources[combo.get_active()][0])
+
     def workspace_name_celldata(self, column, cell, model, iter, user_data=None):
         name = model.get_value(iter, 1).get_name()
         cell.set_property('text', str(name))
@@ -280,13 +293,19 @@ class PreferencesEditor:
         day_start = dt.time(day_start / 60, day_start % 60)
         self.day_start.set_time(day_start)
 
-        self.tags = [tag["name"] for tag in runtime.storage.get_tags(autocomplete=True)]
+        self.tags = [tag["name"] for tag in runtime.storage.get_tags(only_autocomplete=True)]
         self.get_widget("autocomplete_tags").set_text(", ".join(self.tags))
 
         self.workspace_mapping = conf.get("workspace_mapping")
         self.get_widget("workspace_list").set_sensitive(self.get_widget("workspace_tracking_name").get_active())
 
 
+        current_source = conf.get("activities_source")
+        for i, (code, label) in enumerate(self.activities_sources):
+            if code == current_source:
+                self.todo_combo.set_active(i)
+
+
     def on_autocomplete_tags_view_focus_out_event(self, view, event):
         buf = self.get_widget("autocomplete_tags")
         updated_tags = buf.get_text(buf.get_start_iter(), buf.get_end_iter(), 0) \
@@ -427,7 +446,7 @@ class PreferencesEditor:
             return False #ignoring unsorted category
 
         #look for dupes
-        categories = runtime.storage.get_category_list()
+        categories = runtime.storage.get_categories()
         for category in categories:
             if category['name'].lower() == new_text.lower():
                 if id == -2: # that was a new category
@@ -449,10 +468,10 @@ class PreferencesEditor:
         category_id = model[path][2]
 
         #look for dupes
-        activities = runtime.storage.get_activities(category_id)
+        activities = runtime.storage.get_category_activities(category_id)
         for activity in activities:
-            if id != activity['id'] and activity['name'].lower() == new_text.lower():
-                if id == -1: # that was a new activity
+            if activity['name'].lower() == new_text.lower():
+                if id == -1: # that was a new category
                     self.activity_store.remove(model.get_iter(path))
                 self.select_activity(activity['id'])
                 return False
